searchNodes=[{"doc":"Main API for DeltaCheck, a testing toolkit for making assertions on database writes.","ref":"DeltaCheck.html","title":"DeltaCheck","type":"module"},{"doc":"Tracks database writes made in the given block, and asserts that the resulting delta matches the given pattern. Options :schemas - A list of schemas to track writes for, instead of the default schemas provided in the application configuration. Examples iex&gt; assert_changes ( insert : % User { name : &quot;John Doe&quot; } ) do iex&gt; Accounts . create_user ( %{ name : &quot;John Doe&quot; } ) iex&gt; end","ref":"DeltaCheck.html#assert_changes/3","title":"DeltaCheck.assert_changes/3","type":"macro"},{"doc":"Tracks database writes made in the given block, and asserts that no writes were made. Options :schemas - A list of schemas to track writes for, instead of the default schemas provided in the application configuration. Examples iex&gt; assert_no_changes do iex&gt; Accounts . create_user ( %{ name : nil } ) iex&gt; end","ref":"DeltaCheck.html#assert_no_changes/2","title":"DeltaCheck.assert_no_changes/2","type":"macro"},{"doc":"Compares two snapshots and returns a delta. Examples iex&gt; compare ( iex&gt; %{ User =&gt; %{ 1 =&gt; % User { id : 1 , name : &quot;John Doe&quot; } } } , iex&gt; %{ User =&gt; %{ 1 =&gt; % User { id : 1 , name : &quot;Jane Doe&quot; } } } iex&gt; ) [ update : { % User { id : 1 , name : &quot;Jane Doe&quot; } , name : { &quot;John Doe&quot; , &quot;Jane Doe&quot; } } ]","ref":"DeltaCheck.html#compare/2","title":"DeltaCheck.compare/2","type":"function"},{"doc":"Returns the name of the primary key of the schema. Examples iex&gt; get_primary_key! ( User ) :id","ref":"DeltaCheck.html#get_primary_key!/1","title":"DeltaCheck.get_primary_key!/1","type":"function"},{"doc":"Returns all Ecto schemas for the given application. Examples iex&gt; get_schemas ( :my_app ) [ Foo , Bar , Baz ]","ref":"DeltaCheck.html#get_schemas/1","title":"DeltaCheck.get_schemas/1","type":"function"},{"doc":"Returns a snapshot of the database. Options :schemas - A list of schemas to include in the snapshot, instead of the default schemas provided in the application configuration. Examples iex&gt; { :ok , %{ id : id } = user } = Accounts . create_user ( %{ name : &quot;John Doe&quot; } ) iex&gt; snapshot ( schemas : [ User ] ) %{ User =&gt; %{ id =&gt; user } }","ref":"DeltaCheck.html#snapshot/1","title":"DeltaCheck.snapshot/1","type":"function"},{"doc":"Tracks database writes made in the given function, and returns a tuple with the containing the return value of the function and a database delta. Options :schemas - A list of schemas to track writes for, instead of the default schemas provided in the application configuration. Examples iex&gt; { { _ , user } , _ } = track_changes ( fn -&gt; iex&gt; Accounts . create_user ( %{ name : &quot;John Doe&quot; } ) iex&gt; end ) { { :ok , user } , [ insert : user ] }","ref":"DeltaCheck.html#track_changes/2","title":"DeltaCheck.track_changes/2","type":"function"},{"doc":"","ref":"DeltaCheck.html#t:delta/0","title":"DeltaCheck.delta/0","type":"type"},{"doc":"","ref":"DeltaCheck.html#t:snapshot/0","title":"DeltaCheck.snapshot/0","type":"type"},{"doc":"A snapshot strategy is a function, snapshot/2 , that takes an Ecto repo and a list of Ecto schemas, and returns a snapshot of the database in the expected format. The default implementation is DeltaCheck.SnapshotStrategy.RepoAll , but you can configure DeltaCheck to use your own custom snapshot strategy in the application environment: Application . put_env ( :delta_check , :snapshot_strategy , MySnapshotStrategy )","ref":"DeltaCheck.SnapshotStrategy.html","title":"DeltaCheck.SnapshotStrategy","type":"behaviour"},{"doc":"Returns a snapshot of the given Ecto schemas, using the given Ecto repo. Here's an example of the expected format of the snapshot: %{ SchemaA =&gt; %{ 1 =&gt; % SchemaA { field : &quot;foo&quot; , id : 1 } } , SchemaB =&gt; %{ 1 =&gt; % SchemaB { field : &quot;bar&quot; , id : 1 } , 2 =&gt; % SchemaB { field : &quot;baz&quot; , id : 2 } } }","ref":"DeltaCheck.SnapshotStrategy.html#c:snapshot/2","title":"DeltaCheck.SnapshotStrategy.snapshot/2","type":"callback"},{"doc":"The default snapshot strategy for DeltaCheck. This produces the snapshot by running Ecto.Repo.all for all provided schemas.","ref":"DeltaCheck.SnapshotStrategy.RepoAll.html","title":"DeltaCheck.SnapshotStrategy.RepoAll","type":"module"},{"doc":"Copyright (c) 2023 Andreas Geffen Lundahl Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"},{"doc":"Write thorough tests by making assertions on exactly what you insert, update and delete in the database. Here's an example: test &quot;create a user&quot; do assert_changes ( insert : % User { name : &quot;John Doe&quot; } ) do Accounts . create_user ( %{ name : &quot;John Doe&quot; } ) end end DeltaCheck provides a concise API for tracking and asserting on what changes your code makes and doesn't make in the database. Use it to make sure: the entries you expect to be inserted are inserted, the fields you expect to be updated are updated with their expected values, the entries you expect to be deleted are deleted, and nothing else happens in the database.","ref":"readme.html","title":"DeltaCheck","type":"extras"},{"doc":"Add :delta_check to your mix.exs : { :delta_check , &quot;~&gt; 0.1&quot; } In your test_helper.exs , let DeltaCheck know which repo and schemas to use by default: Application . put_all_env ( delta_check : [ repo : MyApp.Repo , schemas : DeltaCheck . get_schemas ( :my_app ) ] ) DeltaCheck.get_schemas/1 will find all Ecto schemas in your application. But you can of course also provide them manually, e.g.: schemas: [MyApp.Foo, MyApp.Bar] . Finally, if you want to, you can add DeltaCheck to your case templates, like MyApp.DataCase and MyApp.ConnCase : import DeltaCheck","ref":"readme.html#installation","title":"DeltaCheck - Installation","type":"extras"},{"doc":"In your tests, you'll primarly use DeltaCheck.assert_changes/3 , DeltaCheck.assert_no_changes/2 and DeltaCheck.track_changes/2 . Since both assert_changes and assert_no_changes are macros that build upon track_changes , let's start with an explanation of the latter. Deltas and track_changes track_changes takes and invokes a function, and returns a tuple containing the return value of the function and a database delta. For example: { return_value , delta } = track_changes ( fn -&gt; Accounts . delete_user_by_id ( 1 ) Accounts . create_user ( %{ name : &quot;John Doe&quot; } ) :some_return_value end ) assert return_value == :some_return_value assert [ insert : % User { name : &quot;John Doe&quot; } , delete : % User { id : 1 } ] = delta The delta is produced by comparing snapshots of the database before and after the function invocation. The delta is a keyword list, where the keys are :insert , :update or :delete , and the values contain the respective schema structs. [ insert : % User { name : &quot;New user&quot; } , update : { % User { id : ^ updated_user_id } , name : { &quot;Old name&quot; , &quot;New name&quot; } } , delete : % User { id : ^ deleted_user_id } ] :insert s always come first, then :update s and then :delete s. Furthermore, the delta is ordered by schema name and finally the primary key. :update is a little bit different, since it doesn't only contain the schema struct of the entry that was updated. Instead, it's a tuple where the first item is the schema struct, and the second item is a keyword list with the fields that where updated. This way you can assert that only the fields you expected to be updated were updated. track_changes should cover all your database assertion needs. But to make your tests just a little bit cleaner, two macros are also provided. assert_changes and assert_no_changes Most uses of track_changes will follow the same pattern, where the delta is bound to something called delta , which is then asserted on like this: assert [...] = delta . To make this pattern a little bit cleaner, DeltaCheck provides assert_changes , which does both things at once: assert_changes ( insert : % User { name : &quot;John Doe&quot; } ) do Accounts . create_user ( %{ name : &quot;John Doe&quot; } ) end # Which is equivalent to: { _ , delta } = track_changes ( fn -&gt; Accounts . create_user ( %{ name : &quot;John Doe&quot; } ) end ) assert [ insert : % User { name : &quot;John Doe&quot; } ] = delta Likewise, when you need to assert that nothing happened in the database, assert_no_changes is helpful: assert_no_changes do Accounts . create_user ( %{ name : nil } ) end # Which is equivalent to: { _ , delta } = track_changes ( fn -&gt; Accounts . create_user ( %{ name : nil } ) end ) assert [ ] = delta With that said, there are use cases for track_changes , where assert_changes and assert_no_changes won't work. One example is when you need to dynamically make assertions on the delta: { _ , delta } = track_changes ( fn -&gt; for _ &lt;- 1 .. 1_000 do Accounts . create_user ( %{ name : &quot;John Doe&quot; } ) end end ) assert Enum . count ( delta ) == 1_000","ref":"readme.html#usage","title":"DeltaCheck - Usage","type":"extras"},{"doc":"Tables without a primary key DeltaCheck currently only works with Ecto schemas that have a primary key defined. This is not a fundamental limitation, though; a fix is being worked on. Performance DeltaCheck generates deltas by taking snapshots of the database and comparing them. This means that DeltaCheck will take a lot of snapshots throughout your test suite, which depending on your application might be a performance problem. If you have less than roughly 20 Ecto schemas defined in your application, the performance penalty is likely going to be negligible. But if you have more than that, it might be worthwhile to configure DeltaCheck to only use a subset of your schemas, or explicitly provide the relevant schemas for each test. Alternatively, you can configure DeltaCheck to use a custom DeltaCheck.SnapshotStrategy , which suits your application better than the default DeltaCheck.SnapshotStrategy.RepoAll .","ref":"readme.html#caveats","title":"DeltaCheck - Caveats","type":"extras"},{"doc":"DeltaCheck is released under the MIT license. See the LICENSE file for more information.","ref":"readme.html#license","title":"DeltaCheck - License","type":"extras"},{"doc":"DeltaCheck is brought to you by Striver , a development consultancy in Sweden. Let us know if we can help you with your Elixir project.","ref":"readme.html#about-striver","title":"DeltaCheck - About Striver","type":"extras"}]